
BASE="https://api.calendly.com"
WEBHOOK=f"{BASE}/webhook_subscriptions"
USERS=f"{BASE}/users"
ME=f"{USERS}/me"
EVENT_TYPE=f"{BASE}/event_types"
EVENTS=f"{BASE}/scheduled_events"
WEBHOOK_SUBSCRIPTIONS=f"{BASE}/webhook_subscriptions"
SCHEDULING_LINKS=f"{BASE}/scheduling_links"
ORGANIZATIONS=f"{BASE}/organizations/"
ORGANIZATION_MEMBERSHIPS=f"{BASE}/organization_memberships"
DATA_COMPLIANCE=f"{BASE}/data_compliance/deletion/invitees"

OAUTH_BASE_URL = "https://auth.calendly.com/oauth"
OAUTH_AUTHORIZE_URL = f"{OAUTH_BASE_URL}/authorize"
OAUTH_TOKEN_URL = f"{OAUTH_BASE_URL}/token"
OAUTH_REVOKE_URL = f"{OAUTH_BASE_URL}/revoke"
OAUTH_INTROSPECT_URL = f"{OAUTH_BASE_URL}/introspect"

